package as.interactive.opencms.widgets;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.apache.commons.logging.Log;
import org.opencms.file.CmsFile;
import org.opencms.file.CmsObject;
import org.opencms.main.CmsException;
import org.opencms.main.CmsLog;
import org.opencms.widgets.CmsSelectWidget;
import org.opencms.widgets.CmsSelectWidgetOption;
import org.opencms.widgets.I_CmsWidget;
import org.opencms.widgets.I_CmsWidgetDialog;
import org.opencms.widgets.I_CmsWidgetParameter;
import org.opencms.xml.content.CmsXmlContent;
import org.opencms.xml.content.CmsXmlContentFactory;
import org.opencms.xml.content.CmsXmlContentValueSequence;

/**
 * @author Stefan Uldum Grinsted <sug@interactive.as>
 *
 * This widget generates a selectbox with values from a xml-content file from
 * a specified location.
 * 
 * The configuration string for this widget must only contain the full vfs-path 
 * to the xmlcontent file, that contains the options for the widget.
 * The xmlcontent file must use the schema supplied with this module:
 *  -  widgetoptions.xsd
 * 
 * See the files in examples/ for a detailes examples of usage.
 * 
 * Notice: The widget must be registered in WEB-INF/config/opencms-vfs.xml under
 * <code>
 * &lt;widgets&gt;...&lt;/widgets&gt;
 * </code>
 * This is done by adding the following line:
 * <code>
 * &lt;widget class="as.interactive.opencms.widgets.CustomValuesSelectWidget" alias="CustomValuesSelectWidget" /&gt;
 * </code>
 */
public class CustomValuesSelectWidget extends CmsSelectWidget {
	private static final Log LOG = CmsLog.getLog(CustomValuesSelectWidget.class);
	static {
		LOG.info(CustomValuesSelectWidget.class + " loaded");
	}
	
	private static final String OPTION_ELEMENT = "Option";
	private static final String VALUE_ELEMENT = "Value";
	private static final String TEXT_ELEMENT = "Text";
	
	private CmsXmlContent xmlcontent;
	private Locale locale;
	private CmsObject cms;
	
	/**
	 * 
	 */
	public CustomValuesSelectWidget() {
		super();
	}
	
	/**
	 * @param configuration
	 */
	public CustomValuesSelectWidget(String configuration) {
		super(configuration);
	}
	
	
	/* (non-Javadoc)
	 * @see org.opencms.widgets.A_CmsSelectWidget#parseSelectOptions(org.opencms.file.CmsObject, org.opencms.widgets.I_CmsWidgetDialog, org.opencms.widgets.I_CmsWidgetParameter)
	 */
	protected List parseSelectOptions(CmsObject cms,
			I_CmsWidgetDialog widgetDialog, I_CmsWidgetParameter param) {
		locale = cms.getRequestContext().getLocale();
		this.cms = cms;
		
		LOG.debug("Entered parseSelectOptions()");
		LOG.debug("getConfiguration() contains: " + getConfiguration());
		
		List options = new ArrayList();
		try {
			// Load the xmlcontent containing the options 
			CmsFile xmlfile = CmsFile.upgrade(cms.readResource(getConfiguration()), cms);
			xmlcontent = CmsXmlContentFactory.unmarshal(cms, xmlfile);
			
			// loop through the options
			CmsXmlContentValueSequence seq = xmlcontent.getValueSequence("Option", locale);
			int count = seq.getElementCount();
			for (int i = 1; i <= count; i++) {
				String value = getValue(VALUE_ELEMENT, i);
				String text = getValue(TEXT_ELEMENT, i);
				CmsSelectWidgetOption option = new CmsSelectWidgetOption(value, false, text);
				options.add(option);
			}
		} catch (CmsException e) {
			CmsSelectWidgetOption option = new CmsSelectWidgetOption("error", true, "Error: Could not load the options from '" + getConfiguration() + "'");
			options.add(option);
		}
		
		LOG.debug("Returning: " + options.toString());
		return options;
	}
	
	/**
	 * Returns the value of the given elementName from the given number of a
	 * sequence. If the element could not be found, null is returned.
	 * 
	 * @param elementName
	 *            the element name
	 * @param number
	 *            the seuence number of the OPTION_ELEMENT
	 * @return the value of the the elementName if found, null otherwise.
	 */
	private String getValue(String elementName, int number) {
		String value = null;
		String elementPath = OPTION_ELEMENT + "[" + number +"]/" + elementName;
		if (xmlcontent.hasValue(elementPath, locale)) {
			value = xmlcontent.getValue(elementPath, locale).getStringValue(cms);
		}
		return value;
	}
	
	/* (non-Javadoc)
	 * @see org.opencms.widgets.CmsComboWidget#newInstance()
	 */
	public I_CmsWidget newInstance() {
		return new CustomValuesSelectWidget(getConfiguration());
	}
}
